<?php

namespace Deployer;

use Symfony\Component\Yaml\Yaml;

require __DIR__ . '/vendor/autoload.php';

require 'recipe/common.php';
require 'recipe/phinx.php';

set('default_stage', 'preprod');

// Project repository
set('repository', 'git@bitbucket.org:bydedi/shop-by-dedi.git');

// Force to use php 7.2
set("bin/php", function () {
    return locateBinaryPath('php7.2');
});

// Hosts
host('preprod')
    ->stage('preprod')
    ->user('shopbydedi')
    ->hostname('preprod.dediagency.net')
    ->set('deploy_path', '/home/shopbydedi/shopbydedi')
    ->set('branch', 'preprod')
;
host('dev')
    ->stage('dev')
    ->user('shopbydedi')
    ->hostname('192.168.1.200')
    ->set('deploy_path', '/home/shopbydedi/www')
    ->set('branch', 'master-refonte')
;

// List of paths which need to be deleted in release after updating code.
set('clear_paths', [
    'storage/php',
    'storage/smarty',
]);

/**
 *
 * /!\ Ne pas modifier après
 *
 */

// [Optional] Allocate tty for git clone. Default value is false.
set('git_tty', true);

set('ssh_multiplexing', true);

set('phinx_path', function () {
    return get('bin/php') . ' vendor/bin/phinx';
}
);

// Shared files/dirs between deploys 
set('shared_files', [
    '.env'
]);

set('shared_dirs', [
    'storage',
    'media',
    'web/media',
]);

// List of dirs which must be writable for web server.
set('writable_dirs', [
    'storage',
    'media',
    'web/media',
]);

set('allow_anonymous_stats', false);

// Tasks
desc('Deploy your project');
task('deploy', [
    'deploy:info',
    'deploy:prepare',
    'deploy:lock',
    'deploy:release',
    'deploy:update_code',
    'deploy:shared',
    'deploy:writable',
    'deploy:vendors',
    'deploy:clear_paths',
    'deploy:symlink',
    'deploy:unlock',
    'cleanup',
]);

after('deploy', 'success');

task('reload:php-fpm', function () {
    run('sudo /usr/sbin/service php7.2-fpm reload');
});

after('deploy:symlink', 'reload:php-fpm');

task('composer:optimize', function () {
    run('cd {{release_path}} && composer dump-autoload --optimize');
})->desc('Optimize loading composer');

task('cache:clear', function () {
    run('cd {{release_path}} && make cache-clear');
})->desc('Clear Cache');

// [Optional] If deploy fails automatically unlock.
after('deploy:failed', 'deploy:unlock');


after('cleanup', 'upload:media:noimage');

after('cleanup', 'phinx:migrate');

before('phinx:migrate', 'install:configure');

task('upload:media:noimage', function () {
    if (test('[ -s {{release_path}}/media/no_image.png ]') == false) {
        upload(__DIR__ . "/media/no_image.png", '{{release_path}}/media/');
    } else {
        writeln('<info>Default image is already installed</info>');
    }
});

task('install:configure', function () {

    if (test('[ -s {{release_path}}/.env ]') == false) {
        invoke('install:configure:app');
    } else {
        writeln('<info>App is already configured</info>');
    }

    if (test('[ -s {{release_path}}/phinx.php ]') == false) {
        invoke('install:configure:db');
    } else {
        writeln('<info>Database is already configured</info>');
    }

})->desc('Ask answer to configure project');

task('install:configure:app', function () {

    writeln('<info>Configure App</info>');
    $app_url = ask('App url');
    $environnement = askChoice('Environnement', ['local', 'dev', 'preprod', 'prod']);
    $debug = askConfirmation('Enable debug ?');

    run('cp {{release_path}}/env.example {{release_path}}/.env');

    // newline
    run ('echo "" >> {{release_path}}/.env');
    run ('echo "# App" >> {{release_path}}/.env');
    run ('echo "APP_URL='.$app_url.'" >> {{release_path}}/.env');
    run ('echo "APP_ENV='.$environnement.'" >> {{release_path}}/.env');
    run ('echo "APP_DEBUG='.$debug.'" >> {{release_path}}/.env');

})->desc('Ask answer to configure project app');

task('install:configure:db', function () {

    writeln('<info>Configure Database</info>');

    $db_host = ask('Host Database');
    $db_name = ask('Database name');
    $db_user = ask('User Database');
    $db_pass = askHiddenResponse('Password Database');

    run ('echo "# Database" >> {{release_path}}/.env');
    run ('echo "DB_HOST='.$db_host.'" >> {{release_path}}/.env');
    run ('echo "DB_DATABASE='.$db_name.'" >> {{release_path}}/.env');
    run ('echo "DB_USERNAME='.$db_user.'" >> {{release_path}}/.env');
    run ('echo "DB_PASSWORD='.$db_pass.'" >> {{release_path}}/.env');

})->desc('Ask answer to configure project database');